<?php

/*
*
* INFOATCLUSTER TECHNOLABS LLP ("COMPANY") CONFIDENTIAL
* Copyright (c) 2016-2017 INFOATCLUSTER TECHNOLABS LLP, All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains the property of INFOATCLUSTER TECHNOLABS LLP. The intellectual and technical concepts contained
* herein are proprietary to INFOATCLUSTER TECHNOLABS LLP and may be covered by Indian and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
* Dissemination of this information or reproduction of this material is strictly forbidden unless prior written permission is obtained
* from COMPANY.  Access to the source code contained herein is hereby forbidden to anyone except current COMPANY employees, managers or contractors who have executed 
* Confidentiality and Non-disclosure agreements explicitly covering such access.
*
* The copyright notice above does not evidence any actual or intended publication or disclosure  of  this source code, which includes  
* information that is confidential and/or proprietary, and is a trade secret, of  COMPANY.   ANY REPRODUCTION, MODIFICATION, DISTRIBUTION, PUBLIC  PERFORMANCE, 
* OR PUBLIC DISPLAY OF OR THROUGH USE  OF THIS  SOURCE CODE  WITHOUT  THE EXPRESS WRITTEN CONSENT OF COMPANY IS STRICTLY PROHIBITED, AND IN VIOLATION OF APPLICABLE 
* LAWS AND INTERNATIONAL TREATIES.  THE RECEIPT OR POSSESSION OF  THIS SOURCE CODE AND/OR RELATED INFORMATION DOES NOT CONVEY OR IMPLY ANY RIGHTS  
* TO REPRODUCE, DISCLOSE OR DISTRIBUTE ITS CONTENTS, OR TO MANUFACTURE, USE, OR SELL ANYTHING THAT IT  MAY DESCRIBE, IN WHOLE OR IN PART.                
*
*
* @author Mohammed Shayas M K
* 
*/

namespace App\Http\Controllers\Admin;
use Illuminate\Http\Request;
use App\Http\Helper\Helper;
use App\Validators\Admin\AdminPositionValidator;
use App\Validators\Admin\AdminSubPositionValidator;
use Illuminate\Support\Facades\Hash;
use App\Managers\Position\PositionManager;
use App\Managers\SubPosition\SubPositionManager;
use Illuminate\Support\Facades\Mail;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Redirect;
use App\Managers\PositionColumns\PositionColumnsManager;
use App\Managers\Profile\ProfileManager;
use App\Managers\PlayerPositions\PlayerPositionsManager;
use App\Managers\FormationPositions\FormationPositionsManager;
use App\Managers\SchedulePlayers\SchedulePlayersManager;
use App\Managers\PlayerRatings\PlayerRatingsManager;
use App\Managers\Tactics\TacticsManager;

class AdminPositionController extends Controller
{
    public function __construct()
    {
        $this->helper = new Helper();
        $this->positionValidator = new AdminPositionValidator();
        $this->subPositionValidator = new AdminSubPositionValidator();
        $this->positionManager = new PositionManager();
        $this->subPositionManager = new SubPositionManager();
        $this->positionColumnsManager = new PositionColumnsManager();
        $this->profile = new ProfileManager();
        $this->playerPositions = new PlayerPositionsManager();
        $this->formationPositionsManager = new FormationPositionsManager();
        $this->schedulePlayersManager = new SchedulePlayersManager();
        $this->playerRatings = new PlayerRatingsManager();
        $this->tacticsManager = new TacticsManager();
    }
    public function addPosition()
    {
        return view('/admin/position/add');
    }
    
    public function savePosition(Request $request)
    {
        if(!$request->all() || !$request->exists('data'))
        {
            return response()->json(['No Input Found'], 400);
        }
        if(!$request->exists('columns'))
        {
            return response()->json(['Please select columns'], 400);
        }
        $data = $request->get('data');
        $columns = $request->get('columns');
        if(empty($columns))
        {
            return response()->json(['Please select atleast one column'], 400);
        }
        $data = $this->helper->changeSerialzeArrayValues($data, 'create');
        
        $validation = $this->positionValidator->add($data, $columns);
        
        if($validation)
        {
            return response()->json($validation['errors'], 400);
        }
        
        $team = $this->positionManager->add($data, $columns);
        if(!$team)
        {
            return response()->json(['Something went wrong'], 500);
        }
        
        return response()->json(['Team Saved Successfully'], 200);
    }
    
    public function listPosition()
    {
        $positions = $this->positionManager->get();
        return view('/admin/position/list')->with('positions', $positions);
    }
    
    public function editPosition($UUID)
    {
        return view('/admin/position/edit')->with('UUID', $UUID);
    }
    
    public function deletePosition($UUID)
    {
        if(!$position = $this->positionManager->where('UUID', $UUID)->first())
        {
            return response()->json(['Position Not Found'], 500);
        }
        
        $playerColumns = $this->playerPositions->with('player')->where('position_id', $UUID)->get();
        if ($playerColumns->isNotEmpty()) {
               $this->playerPositions->where('position_id', $UUID)->delete();
               
//            $message = '<b>The following players are associated with position ' . $position['name'] . ' . You will have to delete the reference to ' . $position['name'] . ' from these players before you delete this Position.</b>';
//            $message.= "<ul>";
//            foreach($playerColumns AS $column)
//            {
//                $message.= "<li>" . $column->player->name . "</li>";
//            }
//            $message.= "</ul>";
//            
//            return response()->json([$message], 400);
//            exit;
        }
        
        $positionColumns = $this->positionColumnsManager->where('position_id', $UUID)->get();
        if(!empty($positionColumns))
        {
            foreach($positionColumns AS $positionColumn)
            {
                $positionColumnsID = $positionColumn['UUID'];
                $this->formationPositionsManager->where('position_columns_id', $positionColumnsID)->delete();
                
                $schedulePlayers = $this->schedulePlayersManager->where('position_columns_id', $positionColumnsID)->get();
                if(!empty($schedulePlayers))
                {
                    foreach($schedulePlayers AS $schedulePlayer)
                    {
                        $schedulePlayerID = $schedulePlayer['UUID'];
                        $this->playerRatings->where('scheduled_players_id', $schedulePlayerID)->delete();
                    }
                }
                
                $this->schedulePlayersManager->where('position_columns_id', $positionColumnsID)->delete();
                $this->tacticsManager->where('position_columns_id', $positionColumnsID)->delete();
            }
        }
        
        $this->positionColumnsManager->where('position_id', $UUID)->delete();
        
        $delete = $this->positionManager->deleteData($UUID);
        if(!$delete)
        {
            return response()->json(['Something went wrong'], 500);
        }
        return response()->json(['Position Deleted Successfully'], 200);
    }
    
    public function currentPosition($UUID)
    {
        $team = $this->positionManager->where('UUID', $UUID)->first()->toArray();
        return response()->json($team,200);
    }
    
    public function updatePosition(Request $request, $UUID)
    {
        if(!$request->all() || !$request->exists('data'))
        {
            return response()->json(['No Input Found'], 400);
        }
        if(!$request->exists('columns'))
        {
            return response()->json(['Please select columns'], 400);
        }
        $data = $request->get('data');
        $columns = $request->get('columns');
        if(empty($columns))
        {
            return response()->json(['Please select atleast one column'], 400);
        }
        $data = $this->helper->changeSerialzeArrayValues($data, 'update');
        
        $validation = $this->positionValidator->update($data, $columns, $UUID);
        
        if($validation)
        {
            return response()->json($validation['errors'], 400);
        }
        
        $team = $this->positionManager->updateData($data, $columns, $UUID);
        if(!$team)
        {
            return response()->json(['Something went wrong'], 500);
        }
        
        return response()->json(['Team Updated Successfully'], 200);
    }
    
    
    
    /* Sub Position */
    public function addSubPosition()
    {
        $positions = $this->positionManager->pluck('name', 'UUID');
        return view('/admin/position/sub_add')->with('positions', $positions);
    }
    
    public function saveSubPosition(Request $request)
    {
        if(!$request->all() || !$request->exists('data'))
        {
            return response()->json(['No Input Found'], 400);
        }
        $data = $request->get('data');
        $data = $this->helper->changeSerialzeArrayValues($data, 'create');
        
        $validation = $this->subPositionValidator->add($data);
        
        if($validation)
        {
            return response()->json($validation['errors'], 400);
        }
        
        $team = $this->subPositionManager->add($data);
        if(!$team)
        {
            return response()->json(['Something went wrong'], 500);
        }
        
        return response()->json(['Team Saved Successfully'], 200);
    }
    
    public function listSubPosition()
    {
        $subPositions = $this->subPositionManager->with('position')->get();
        return view('/admin/position/sub_list')->with('subPositions', $subPositions);
    }
    
    public function editSubPosition($UUID)
    {
        $positions = $this->positionManager->pluck('name', 'UUID');
        return view('/admin/position/sub_edit')->with('UUID', $UUID)->with('positions', $positions);
    }
    
    public function currentSubPosition($UUID)
    {
        $team = $this->subPositionManager->where('UUID', $UUID)->first()->toArray();
        return response()->json($team,200);
    }
    
    public function updateSubPosition(Request $request, $UUID)
    {
        if(!$request->all() || !$request->exists('data'))
        {
            return response()->json(['No Input Found'], 400);
        }
        $data = $request->get('data');
        $data = $this->helper->changeSerialzeArrayValues($data, 'update');
        
        $validation = $this->subPositionValidator->update($data, $UUID);
        
        if($validation)
        {
            return response()->json($validation['errors'], 400);
        }
        
        $team = $this->subPositionManager->updateData($data, $UUID);
        if(!$team)
        {
            return response()->json(['Something went wrong'], 500);
        }
        
        return response()->json(['Team Updated Successfully'], 200);
    }
    
    public function getColumnsOfCurrentPosition($positionID)
    {
        $columns = $this->positionColumnsManager->getColumnsOfCurrentPosition($positionID);
        return response()->json($columns, 200);
    }
    
    public function allPositionColumns()
    {
        $columns = $this->positionColumnsManager->with('position')->where('inactive', 0)->where('deleted_at', NULL)->get();
        
        return response()->json($columns, 200);
    }
}